﻿using System;
using System.Collections.Generic;
using System.Timers;
using Ozeki.Media;
using Ozeki.VoIP;

namespace CallQueue
{
    class CallQueueCallHandler
    {
        ICall call;
        Timer timer;
        ICallManager callManager;
        IExtensionContainer extensionContainer;
        List<string> members;

        MediaConnector mediaConnector;
        PhoneCallAudioSender phoneCallAudioSender;
        MP3StreamPlayback mp3Player;

        public CallQueueCallHandler(ICall call, ICallManager callManager, IExtensionContainer extensionContainer, List<string> members)
        {
            timer = new Timer(1000);
            timer.AutoReset = true;
            timer.Elapsed += timer_Elapsed;

            this.members = members;
            this.callManager = callManager;
            this.extensionContainer = extensionContainer;

            this.call = call;
            this.call.CallStateChanged += call_CallStateChanged;

            mediaConnector = new MediaConnector();
            phoneCallAudioSender = new PhoneCallAudioSender();
            mp3Player = new MP3StreamPlayback("test.mp3");
            phoneCallAudioSender.AttachToCall(call);

            mediaConnector.Connect(mp3Player, phoneCallAudioSender);
        }

        public void Start()
        {
            if (call.CallState.IsCallEnded())
            {
                OnCompleted();
                return;
            }

            timer.Start();
        }

        private void OnCompleted()
        {
            mediaConnector.Dispose();
            mp3Player.Dispose();

            var handler = Completed;
            if (handler != null)
                handler(this, EventArgs.Empty);
        }
        
        public event EventHandler Completed;

        void timer_Elapsed(object sender, ElapsedEventArgs e)
        {
            if (!call.CallState.IsInCall())
                return;

            var memb = new List<string>(members);

            foreach (var activeSession in callManager.ActiveSessions)
            {
                var callee = activeSession.CalleeInfo.Owner.ExtensionID;
                var caller = activeSession.CallerInfo.Owner.ExtensionID;

                memb.Remove(callee);
                memb.Remove(caller);
            }

            foreach (var member in memb)
            {
                if (extensionContainer.GetExtension(member) != null)
                {
                    call.BlindTransfer(member);
                }
            }
        }

        void call_CallStateChanged(object sender, CallStateChangedArgs e)
        {
            if (e.State == CallState.Answered)
            {
                mp3Player.Start();
            }

            if (e.State.IsRinging())
            {
                call.Answer();
            }
            
            else if (e.State.IsCallEnded())
            {
                OnCompleted();
            }
        }
    }
}
